"use strict";
module.exports = function (RED) {
    function DeDuplicate(config) {
        RED.nodes.createNode(this, config);
        this.keyfield = config.keyfield;
        var node = this;

       
        function isEmptyObject(obj) {
			return !Object.keys(obj).length;
	    }
     
		function lookUpKey(key,data){

			var list=Object.keys(data[0]);
			var isMatching =false; 
			var matchingKeyInData;    

			for(var i in list){
				
				if (list[i].trim() == key.trim()){ 
							
					isMatching=true;
					break;
				}
			}
			return isMatching;
		}


		function doesExists(jsonArray, key, value) { 
			return jsonArray.find(obj=>obj[key]==value)
		}
	

        this.on('input', function (msg) {
            if (node.cache === undefined) {
                node.cache = [];
            }
					
            var key = node.keyfield;
			
			var data = msg.payload;
            
           
            if(isEmptyObject(data))
			{
				node.error('Empty Input received in Deduplicate');
				return;
			}
            else{

               var jsonStr=JSON.stringify(data);               
                                              
                try {                                

				 var parsedJson= JSON.parse(jsonStr); 			
				
				} catch (e) {
						
					node.error({"JSON Error in JSON Deduplicate":e});		
					return;
				} 

            }

            
            var isMatchingKey=lookUpKey(key,data);

            var errorMessage= "Key Field Not Found in data."

			if(!isMatchingKey){
           
				node.error(errorMessage);			
				return;

			}
			
			var uniqueArray = [];
			for(var i = 0; i < data.length; i++) {
				var obj = data[i];
				var value = doesExists(uniqueArray,key,obj[key]);

				if(value == null)
				{
					uniqueArray.push(obj);
				}
			}
			msg.payload = uniqueArray;
            node.send(msg);
			return;
        });
    }
    RED.nodes.registerType("JSONDeduplicate", DeDuplicate);
};